## target distribution ##
g=function(x){
  if(x<=10 & x>=-10) return(exp(-(1/sin(x))^10/4)) else return(0)
}

## true density (include normalizing constant) ##
true=function(x){exp(-(1/sin(x))^10/4)/4.488231}

## requires package "msm"
library(msm)

## Ordinary MCMC ##
x=NULL;N=10000
u=runif(N,0,1);
x[1]=5;
for(i in 1:N){
  tmp=rtnorm(1,x[i],0.3,lower=-10,upper=10);
  if(u[i]<=g(tmp)*dtnorm(x[i],tmp,0.3,lower=-10,upper=10)/g(x[i])/dtnorm(tmp,x[i],0.3,lower=-10,upper=10)) x[i+1]=tmp else x[i+1]=x[i]
}
hist(x,xlim=c(-10,10),freq=F,breaks=300,main="",xlab="x",ylab=expression(paste(pi,"(x)")))
true=function(x){exp(-(1/sin(x))^10/4)/4.488231}
curve(true,-10,10,ylim=c(0,0.2),add=T,lwd=2)

## SMSS algorithms ##
multi=function(x,s)
{
  length(which(x==s))
}

## SMS-MWG ##
x=NULL;N=10000;K=3
S=matrix(0,N+1,K);
u=runif(N,0,1);
x[1]=5;S[1,]=x[1]
for(i in 1:N){
  id=sample(1:K,1);
  tmp=S[i,];tmp[id]=rtnorm(1,S[i,id],0.3,lower=-10,upper=10);
  if(u[i]<=multi(x[i],tmp)*multi(tmp[id],tmp)*dtnorm(S[i,id],tmp[id],0.3,lower=-10,upper=10)/multi(x[i],S[i,])/multi(S[i,id],S[i,])/dtnorm(tmp[id],S[i,id],0.3,lower=-10,upper=10)) S[i+1,]=tmp else S[i+1,]=S[i,]
  prop=sapply(S[i+1,],g);
  x.id=sample(1:K,1,prob=prop);
  x[i+1]=S[i+1,x.id];
}
hist(x,xlim=c(-10,10),freq=F,breaks=300,main="",ylim=c(0,0.4),xlab="",ylab=expression(paste(pi,"(x)")))
true=function(x){exp(-(1/sin(x))^10/4)/4.488231}
curve(true,-10,10,add=T,lwd=2)

## SMS-CBMC ##
x=NULL;N=10000;K=3
S=matrix(0,N+1,K);
u=runif(N,0,1);
x[1]=5;S[1,]=x[1]
for(i in 1:N){
  id=sample(1:K,1);
  tmp=S[i,];tmp[id]=rtnorm(1,S[i,id],0.3,lower=-10,upper=10);
  if(u[i]<=sum(sapply(tmp,g))*multi(tmp[id],tmp)*dtnorm(S[i,id],tmp[id],0.3,lower=-10,upper=10)/sum(sapply(S[i,],g))/multi(S[i,id],S[i,])/dtnorm(tmp[id],S[i,id],0.3,lower=-10,upper=10)) {
    S[i+1,]=tmp;
    prop=sapply(S[i+1,],g);
    x.id=sample(1:K,1,prob=prop);
    x[i+1]=S[i+1,x.id];
  } else {S[i+1,]=S[i,];x[i+1]=x[i]}
}
hist(x,xlim=c(-10,10),freq=F,breaks=300,main="",ylim=c(0,0.4),xlab="x",ylab=expression(paste(pi,"(x)")))
true=function(x){exp(-(1/sin(x))^10/4)/4.488231}
curve(true,-10,10,add=T,lwd=2)


## R-SMS-CBMC ## 
x=NULL;N=10000;K=3
S=matrix(0,N+1,K);
u=runif(N,0,1);
x[1]=5;S[1,]=x[1]
for(i in 1:N){
  id=sample(1:K,1);
  tmp=S[i,];tmp[id]=rtnorm(1,S[i,id],0.3,lower=-10,upper=10);
  if(u[i]<=sum(sapply(tmp,g))*multi(tmp[id],tmp)*dtnorm(S[i,id],tmp[id],0.3,lower=-10,upper=10)/sum(sapply(S[i,],g))/multi(S[i,id],S[i,])/dtnorm(tmp[id],S[i,id],0.3,lower=-10,upper=10)) S[i+1,]=tmp else S[i+1,]=S[i,]
  prop=sapply(S[i+1,],g);
  x.id=sample(1:K,1,prob=prop);
  x[i+1]=S[i+1,x.id];
}
hist(x,xlim=c(-10,10),freq=F,breaks=300,main="",ylim=c(0,0.4),xlab="x",ylab=expression(paste(pi,"(x)")))
true=function(x){exp(-(1/sin(x))^10/4)/4.488231}
curve(true,-10,10,add=T,lwd=2)

## Parallel Tempering ##
N=10^4
swap=0.5
K=3
sigma=1
inver_t=seq(1,0,length.out=K)
x=matrix(0,K,N+1)
x[,1]=5;
iter=1
while(iter<=N){
  u=runif(1,0,1)
  if (u<=swap){
    for (i in 1:K){
      tmp=rtnorm(1,x[i,iter],sigma,lower=-10,upper=10)
      if(runif(1,0,1)<=g(tmp)^inver_t[i]*dtnorm(x[i,iter],tmp,sigma,lower=-10,upper=10)/(g(x[i,iter])^inver_t[i])/dtnorm(tmp,x[i,iter],sigma,lower=-10,upper=10)){
          x[i,iter+1]=tmp
      } else {
          x[i,iter+1]=x[i,iter]
      }
    }
    iter=iter+1
  } else {
      x[,iter+1]=x[,iter]
      j=sample(K-1,1)
      if(runif(1,0,1)<=g(x[j+1,iter])^inver_t[j]*g(x[j,iter])^inver_t[j+1]/g(x[j,iter])^inver_t[j]/g(x[j+1,iter])^inver_t[j+1]){
        temp=x[j,iter+1]
        x[j,iter+1]=x[j+1,iter+1]
        x[j+1,iter+1]=temp
    }
    iter=iter+1
  }
}
fit=density(x[1,],from=-10, to=10,n=200)
truth=sapply(fit$x,true)
tv=sum(abs(truth-fit$y))/2*0.1

# calculate iat
myacf=acf(x[1,],lag.max=500,plot=FALSE)
rho=myacf$acf[-1]
iat=0.5+sum(rho*(1-1:500/500))
iat

hist(x[1,],xlim=c(-10,10),freq=F,breaks=300,main="",xlab="x",ylab=expression(paste(pi,"(x)")))
true=function(x){exp(-(1/sin(x))^10/4)/4.488231}
curve(true,-10,10,ylim=c(0,0.2),add=T,lwd=2)


## EMC ##
crossover=function(x,y,k){
  base=10^(k-1)
  res1=floor(x*base)/base+y-floor(y*base)/base
  res2=floor(y*base)/base+x-floor(x*base)/base
  return(list(res1,res2))
}

n=10^4
mutation=0.3
N=3
sigma=1
inver_t=seq(1,0,length.out=N)
x=matrix(0,N,n+1)
x[,1]=5;
iter=1
while(iter<=n){
  u=runif(1,0,1)
  if (u<=mutation){
    for (z in 1:N){
      tmp=rtnorm(1,x[z,iter],sigma,lower=-10,upper=10)
      if(runif(1,0,1)<=g(tmp)^inver_t[z]*dtnorm(x[z,iter],tmp,sigma,lower=-10,upper=10)/(g(x[z,iter])^inver_t[z])/dtnorm(tmp,x[z,iter],sigma,lower=-10,upper=10)){
          x[z,iter+1]=tmp
      } else {
          x[z,iter+1]=x[z,iter]
      }
    }
    iter=iter+1
  } else {
      #crossover
      x[,iter+1]=x[,iter]
      fitness=sapply(x[,iter],g)
      pool=1:N
      i=sample(pool,1,prob=fitness)
      j=sample(pool[pool!=i],1)
      res_co=crossover(x[i,iter],x[j,iter],sample(2,1))
      yi=res_co[[1]]
      yj=res_co[[2]]
      zx=sum(fitness)
      zy=zx-g(x[i,iter])-g(x[j,iter])+g(yi)+g(yj)
      if(zy!=0 & runif(1,0,1)<=(g(yi)/g(x[i,iter]))^inver_t[i]*(g(yj)/g(x[j,iter]))^inver_t[j]*(g(yi)+g(yj))/zy*zx/(g(x[i,iter])+g(x[j,iter]))){
        x[i,iter+1]=yi
        x[j,iter+1]=yj
    }
    iter=iter+1
  }
    #exchange
  s=sample(N,1)
  if(s==1) t=s+1 else{
      t=ifelse(s==N,s-1,sample(c(s-1,s+1),1))
  }
  if(runif(1,0,1)<=(g(x[s,iter])/g(x[t,iter]))^(inver_t[t]-inver_t[s])){
      temp=x[t,iter]
      x[t,iter]=x[s,iter]
      x[s,iter]=temp
  }
}
fit=density(x[1,],from=-10, to=10,n=200)
truth=sapply(fit$x,true)
tv=sum(abs(truth-fit$y))/2*0.1

# calculate iat #
myacf=acf(x[1,],lag.max=500,plot=FALSE)
rho=myacf$acf[-1]
iat=0.5+sum(rho*(1-1:500/500))
iat

hist(x[1,],xlim=c(-10,10),freq=F,breaks=300,main="",xlab="x",ylab=expression(paste(pi,"(x)")))
true=function(x){exp(-(1/sin(x))^10/4)/4.488231}
curve(true,-10,10,ylim=c(0,0.2),add=T,lwd=2)

