# The dataset is inside the pacakage "qvalue", need to run the following two lines of codes to install the package first
# source("http://bioconductor.org/biocLite.R")
# biocLite("qvalue")
library(qvalue)
data(hedenfalk)
# transform the data to z-values
z=qnorm(1-hedenfalk$p,0,1)


# require package "MCMCpack" and "msm"
library(MCMCpack)
library(msm)


## M-H ##
xi=(max(z)+min(z))/2; R=max(z)-min(z)
ind_xi=function(t){
	res=ifelse(t>0 & t<1, 1,0)
	return(res)
}
target_logx=function(x){  
	prior=ind_xi(x[1])*dnorm(x[2],mean=xi,sd=R)*dinvgamma(x[3],2,2)
	lik=sapply(z, function(u){(1-x[1])*dnorm(u,0,1)+x[1]*dnorm(u,x[2],sqrt(x[3]))})
	return(log(prior)+sum(log(lik)))
}

sig_mu=0.2;sig_sg=0.2;sig_xi=0.1
N=2e4
u=runif(N,0,1) #update all three components at the same time
theta=matrix(0,N+1,3)
theta[1,]=c(0.5,1,1)
t_logx=target_logx(theta[1,])
for(i in 1:N){
	temp=NULL
	temp[1]=rtnorm(1,theta[i,1],sd=sig_xi,lower=0,upper=1)
	temp[2]=rnorm(1,theta[i,2],sd=sig_mu)
	temp[3]=rlnorm(1,log(theta[i,3]),sd=sig_sg)
	t_temp=target_logx(temp)
	if(u[i]<=exp(t_temp-t_logx)*dlnorm(theta[i,3],log(temp[3]),sig_sg)*dtnorm(theta[i,1],temp[1],sig_xi,0,1)/dlnorm(temp[3],log(theta[i,3]),sig_sg)/dtnorm(temp[1],theta[i,1],sig_xi,0,1)) {theta[i+1,]=temp;t_logx=t_temp} else theta[i+1,]=theta[i,]
}

par(mfrow=c(1,2))
plot(density(theta[,2],bw=0.1),xlab=expression(mu),main="",ylim=c(0,3),lwd=2)
plot(theta[,2],xlab="",ylab=expression(mu),ylim=c(0.5,2))




## GSMSS ##
inst=function(x){   #x=(xi,mu,sigma2)
	lower=c(0,0.2,exp(-0.6))
	upper=c(1,2,exp(0.8))
	inst=ifelse(all(x>lower) & all(x<upper),1,0)
	return(inst)
}
target_s=function(s){
	temp=apply(s,2,target_logx)
	m.factor=min(temp)
	temp=temp-m.factor
	instf=rev(apply(s,2,inst))
	result=1/k*sum(exp(temp)*instf)
	return(list(result,(-1)*m.factor,temp,instf))
}

N=2e4; k=2
sig_mu=0.2;sig_sg=0.2;sig_xi=0.1
theta=matrix(0,N+1,3)
weight=matrix(0,N+1,k)
theta[1,]=c(0.5,1,1)
s=matrix(rep(theta[1,],k),byrow=F,ncol=k)
t_s=target_s(s)
p=exp(t_s[[3]])*t_s[[4]]
weight[1,]=p
u=runif(N,0,1)
s.list=vector("list",N+1)
s.list[[1]]=s
for(i in 1:N){
	s.id=sample(1:k,1)
	news=s; 
	news[1,s.id]=rtnorm(1,theta[i,1],sd=sig_xi,lower=0,upper=1)
	news[2,s.id]=rnorm(1,theta[i,2],sd=sig_mu)
	news[3,s.id]=rlnorm(1,log(theta[i,3]),sd=sig_sg)
	t_new=target_s(news)
	if(u[i]<exp(t_s[[2]]-t_new[[2]])*t_new[[1]]*dlnorm(theta[i,3],log(news[3,s.id]),sig_sg)*dtnorm(theta[i,1],news[1,s.id],sig_xi,0,1)/t_s[[1]]/dlnorm(news[3,s.id],log(theta[i,3]),sig_sg)/dtnorm(news[1,s.id],theta[i,1],sig_xi,0,1)) {s=news;t_s=t_new;p=exp(t_s[[3]])*t_s[[4]]}
	theta.id=sample(1:k,1,prob=p)
	theta[i+1,]=s[,theta.id]
	weight[i+1,]=p
	s.list[[i+1]]=s
}

par(mfrow=c(1,2))
plot(density(theta[,2],bw=0.1),xlab=expression(mu),main="",ylim=c(0,3),lwd=2)
plot(theta[,2],xlab="",ylab=expression(mu),ylim=c(0.5,2))
